
import os
import logging
import zc.buildout.easy_install
import pkg_resources

logger = zc.buildout.easy_install.logger

required_by = {}

def _log_requirement(ws, req):
    ws = list(ws)
    ws.sort()
    for dist in ws:
        if req in dist.requires():
            req_ = str(req)
            dist_ = str(dist)
            if req_ in required_by and dist_ not in required_by[req_]:
                required_by[req_].append(dist_)
            else:
                required_by[req_] = [dist_]
            logger.debug("  required by %s." % dist)


def enable_dumping_picked_versions(old_get_dist):
    def get_dist(self, requirement, ws, always_unzip):
        dists = old_get_dist(self, requirement, ws, always_unzip)
        for dist in dists:
            if not (dist.precedence == pkg_resources.DEVELOP_DIST or \
                    (len(requirement.specs) == 1 and requirement.specs[0][0] == '==')):
                self.__picked_versions[dist.project_name] = dist.version
            if not (dist.precedence == pkg_resources.DEVELOP_DIST):
                self.__all_picked_versions[dist.project_name] = dist.version
        return dists        
    return get_dist


def dump_picked_versions(old_logging_shutdown, file_name, overwrite):

    def logging_shutdown():

        picked_versions_top = ''
        picked_versions_bottom = ''
        for d, v in sorted(zc.buildout.easy_install.Installer.__all_picked_versions.items()):
            if d in required_by:
                req_ = "\n#Required by:\n#" + "\n#".join(required_by[d]) + "\n"
                picked_versions_bottom += "%s%s==%s\n" % (req_, d, v)
            else:
                picked_versions_top += "%s==%s\n" % (d, v)

        picked_versions = picked_versions_top + picked_versions_bottom

        if file_name is not None:
            if not os.path.exists(file_name):
                print "*********************************************"
                print "Writing picked versions to %s" % file_name
                print "*********************************************"
                open(file_name, 'w').write(picked_versions)
            elif overwrite:
                print "*********************************************"
                print "Overwriting %s" % file_name
                print "*********************************************"
                open(file_name, 'w').write(picked_versions)
            else:    
                print "*********************************************"
                print "Skipped: File %s already exists." % file_name                 
                print "*********************************************"
        else:
            print "*************** PICKED VERSIONS ****************"
            print picked_versions
            print "*************** /PICKED VERSIONS ***************"

        old_logging_shutdown()    
    return logging_shutdown


def install(buildout):

    file_name = 'dump-requirements-file' in buildout['buildout'] and \
                buildout['buildout']['dump-requirements-file'].strip() or \
                None
                
    overwrite = 'overwrite-requirements-file' not in buildout['buildout'] or \
                buildout['buildout']['overwrite-requirements-file'].lower() \
                in ['yes', 'true', 'on']

    zc.buildout.easy_install.Installer.__picked_versions = {}
    zc.buildout.easy_install.Installer.__all_picked_versions = {}
    zc.buildout.easy_install._log_requirement = _log_requirement
    zc.buildout.easy_install.Installer._get_dist = enable_dumping_picked_versions(
                                  zc.buildout.easy_install.Installer._get_dist)
    
    logging.shutdown = dump_picked_versions(logging.shutdown, 
                                            file_name, 
                                            overwrite)
    



""" Implement requirements dumping for as buildout plugin. """

import os
import logging
import zc.buildout.easy_install
import pkg_resources

logger = zc.buildout.easy_install.logger

required_by = {}


def _log_requirement(ws, req):
    ws = list(ws)
    ws.sort()
    for dist in ws:
        if req in dist.requires():
            req_ = str(req)
            dist_ = str(dist)
            if req_ in required_by and dist_ not in required_by[req_]:
                required_by[req_].append(dist_)
            else:
                required_by[req_] = [dist_]
            logger.debug('  required by %s.' % dist)


def enable_dumping_picked_versions(old_get_dist):
    """ Monkey path __get_dist to store requirements. """
    def get_dist(self, requirement, ws, **kwargs):
        dists = old_get_dist(self, requirement, ws, **kwargs)

        for dist in dists:
            if not (dist.precedence == pkg_resources.DEVELOP_DIST) and \
               not kwargs.get('for_buildout_run', False):
                    self.__all_picked_versions[dist.project_name] \
                        = dist.version
        return dists
    return get_dist


def dump_picked_versions(old_logging_shutdown, file_name, overwrite):
    """ Monkey patch logging_shutdown to write requirements file. """
    def logging_shutdown():

        picked_versions_top = ''
        picked_versions_bottom = ''
        picked = zc.buildout.easy_install.Installer \
                                         .__all_picked_versions.items()
        for d, v in sorted(picked):
            if d in required_by:
                req_ = '\n#Required by:\n#' + '\n#'.join(required_by[d]) + '\n'
                picked_versions_bottom += '%s%s==%s\n' % (req_, d, v)
            else:
                picked_versions_top += '%s==%s\n' % (d, v)

        picked_versions = picked_versions_top + picked_versions_bottom

        if file_name is not None:
            if not os.path.exists(file_name):
                print('*********************************************')
                print('Writing picked versions to %s' % file_name)
                print('*********************************************')
                open(file_name, 'w').write(picked_versions)
            elif overwrite:
                print('*********************************************')
                print('Overwriting %s' % file_name)
                print('*********************************************')
                open(file_name, 'w').write(picked_versions)
            else:
                print('*********************************************')
                print('Skipped: File %s already exists.' % file_name)
                print('*********************************************')
        else:
            print('*************** PICKED VERSIONS ****************')
            print(picked_versions)
            print('*************** /PICKED VERSIONS ***************')

        old_logging_shutdown()
    return logging_shutdown


def install(buildout):
    """ Install the requirements buildout extension. """
    file_name = 'dump-requirements-file' in buildout['buildout'] and \
                buildout['buildout']['dump-requirements-file'].strip() or \
                None

    overwrite = 'overwrite-requirements-file' not in buildout['buildout'] or \
                buildout['buildout']['overwrite-requirements-file'].lower() \
                in ['yes', 'true', 'on']

    zc.buildout.easy_install.Installer.__all_picked_versions = {}
    zc.buildout.easy_install._log_requirement = _log_requirement
    zc.buildout.easy_install.Installer._get_dist = \
        enable_dumping_picked_versions(
            zc.buildout.easy_install.Installer._get_dist)

    logging.shutdown = dump_picked_versions(logging.shutdown,
                                            file_name,
                                            overwrite)



# -*- coding: utf-8 -*-
import os
from setuptools import setup, find_packages


def read(*rnames):
    return open(os.path.join(os.path.dirname(__file__), *rnames)).read()

version = '0.2.1'

long_description = (read('README.rst') + '\n' + read('CONTRIBUTORS.rst'))
entry_point = 'buildout.requirements:install'
entry_points = {'zc.buildout.extension': ['ext = %s' % entry_point]}

tests_require = ['zc.buildout', 'zope.testing', 'zc.recipe.egg']

setup(name='buildout.requirements',
      version=version,
      description='Dump buildout picked versions '
      'as a pip compatible requirements.txt file. Fork of buildout.dumprequirements.',
      long_description=long_description,
      classifiers=[
          'Framework :: Buildout',
          'Intended Audience :: Developers',
          'Topic :: Software Development :: Build Tools',
          'Topic :: Software Development :: Libraries :: Python Modules',
      ],
      license='GPL',
      keywords='buildout extension dump picked versions requirements.txt',
      author='Yves Müller',
      author_email='yves.mueller@liqd.de',
      url='https://github.com/liqd/buildout.requirements',
      packages=find_packages(),
      namespace_packages=['buildout'],
      include_package_data=True,
      zip_safe=False,
      install_requires=['setuptools',
                        'zc.buildout'],
      tests_require=tests_require,
      extras_require=dict(tests=tests_require),
      entry_points=entry_points)
